import React from 'react'
import PropTypes from 'prop-types'
import classNames from 'classnames'
import { notification } from 'antd'

import SearchForm from './SearchForm'
import SearchBody from './SearchBody'
import BodyHeader from './BodyHeader'
import BodyPagination from './BodyBodyPaginationation'
import { getStorageBuilder } from './utils'
import Style from './style.less'

class SearchList extends React.PureComponent {
  static propTypes = {
    title: PropTypes.element,
    actions: PropTypes.element,
    search: PropTypes.object,
    body: PropTypes.object,
    pagination: PropTypes.object,
  }

  storage = getStorageBuilder(window.Math.random().toString(36).substr(2))

  state = {
    dataSource: [],
    conditions: {},
    pageConfig: {
      current: 1,
      pageSize: 10,
      total: 0,
    },
  }

  handlePageChange = (page, pageSize) => {
    this.setState(
      (state) => ({
        ...state,
        pageConfig: {
          ...pageConfig,
          current: page,
          pageSize,
        }
      }),
      this.queryData,
    )
  }

  handleSearchSubmit = (conditions) => {
    this.setState(
      (state) => ({
        ...state,
        conditions,
      }),
      this.queryData,
    )
  }

  queryData = () => {
    if (this.props.service) {
      const pageConfigProps = this.props.pagination || {}
      const keyOfPageNumber = pageConfigProps.keyOfPageNumber || 'pageNumber'
      const keyOfPageSize= pageConfigProps.keyOfPageSize || 'pageSize'
      const keyOfTotalSize= pageConfigProps.keyOfTotalSize || 'total'

      const { conditions = {}, pageConfig } = this.state

      this.props.service({
        ...conditions,
        ...{
          [`${keyOfPageNumber}`]: pageConfig.current,
          [`${keyOfPageSize}`]: pageConfig.pageSize,
        },
      }).then((res) => {
        const [dataSource, pageParams] = res || []

        this.setState((state) => ({
          ...state,
          dataSource,
          pageConfig: {
            ...pageConfig,
            total: pageParams[keyOfTotalSize],
          },
        }))
      }).catch((e) => {
        console.error(e)
      })
    }
  }

  render() {    
    const {
      keyOfPageNumber,
      keyOfPageSize,
      keyOfTotalSize,
      ...leftPageParams,
    } = this.props.pagination || {}

    const pageConfig = {
      ...leftPageParams,
      ...this.state.pageConfig,
    }

    return (
      <div className={Style['search-list']}>
        <div className={Style['head']}>
          <SearchForm storage={storage} onSubmit={this.handleSearchSubmit} configs={this.props.search} />
        </div>
        <div className={Style['body']}>
          <BodyHeader storage={storage} title={this.props.title} actions={this.props.actions} />
          <SearchBody storage={storage} dataSource={this.state.dataSource} configs={this.props.body} />
          <BodyPagination storage={storage} onChange={this.handlePageChange} configs={pageConfig} />
        </div>
      </div>
    )
  }
}
